from itchio.User import User

class DownloadKey:
    def __init__(
        self,
        id: int,
        created_at: str,
        downloads: int,
        key: str,
        game_id: int,
        owner: User) -> None:

        self.id = id
        self.created_at = created_at
        self.downloads = downloads
        self.key = key
        self.game_id = game_id
        self.owner = owner

    @staticmethod
    def parse_from_dict(data: dict) -> object:
        return DownloadKey(
            id = data["id"],
            created_at = data["created_at"],
            downloads = data["downloads"],
            key = data["key"],
            game_id = data["game_id"],
            owner = User.parse_from_dict(data["owner"])
        )


import requests

class Session:
    """
    A session object that is used for making requests
    """
    def __init__(self, key: str, url: str = "https://itch.io/api/1"):
        self.key = key
        self.url = f"{url}/{key}/"
        self.session = requests.session()
    
    def get(self, route: str) -> dict:
        """
        Get a certain ressource
        """
        response = self.session.get(f"{self.url}{route}")
        if response.ok:
            return response.json()
        else:
            raise Exception(f"Response of {self.url}{route} was {response.status_code}")


from requests.sessions import session
from itchio.Session import Session
from itchio.Game import Game

class GameCollection:
    """
    Get information about your games
    """
    def __init__(self, session: Session) -> None:
        self.session = session

    def all(self) -> list[Game]:
        data = self.session.get("my-games")
        games = []

        for game in data["games"]:
            games.append(
                Game.parse_from_dict(game, self.session)
            )

        return games

    def get_by_id(self, id: int) -> Game:
        data = self.session.get(f"game/{id}")
        return Game.parse_from_dict(data["game"], self.session)


from itchio.Session import Session


class User:
    """
    An Object representing the User ressource
    """
    def __init__(
        self,
        username: str,
        gamer: bool,
        display_name: str,
        cover_url: str,
        url: str,
        press_user: bool,
        developer: bool,
        id: int,
        session: Session):
        self.username = username
        self.gamer = gamer
        self.display_name = display_name
        self.cover_url = cover_url
        self.url = url
        self.press_user = press_user
        self.developer = developer
        self.id = id
        self.session = session

    def sync(self) -> object:
        """
        Sync this object with the information in the API
        """
        data = self.session.get("me")
        new_user = self.parse_from_dict(data["user"], self.session)
        self.__dict__.update(new_user.__dict__)

    @staticmethod
    def parse_from_dict(data: dict, session: Session) -> object:
        # If there is no 'display_name', the value is the same as the username
        if "display_name" not in data:
            data["display_name"] = data["username"]

        return User(
            username = data["username"],
            gamer = data["gamer"],
            display_name = data["display_name"],
            cover_url = data["cover_url"],
            url = data["url"],
            press_user = data["press_user"],
            developer = data["developer"],
            id = data["id"],
            session = session
        )


from itchio.Session import Session
from itchio.User import User

class UserCollection:
    """
    Get User info
    """
    def __init__(self, session: Session) -> None:
        self.session = session

    def me(self) -> User:
        """
        Get your account / The owner of the token
        """
        data = self.session.get("me")
        return User.parse_from_dict(data["user"], self.session)


from typing import Union


class Purchase:
    def __init__(
        self,
        donation: Union[bool, None],
        id: int,
        email: str,
        created_at: str,
        source: str,
        currency: str,
        price: str,
        sale_rate: int,
        game_id: int) -> None:

        self.donation = donation
        self.id = id
        self.email = email
        self.created_at = created_at
        self.source = source
        self.currency = currency
        self.price = price
        self.sale_rate = sale_rate
        self.game_id = game_id

    @staticmethod
    def parse_from_dict(data: dict):
        return Purchase(
            donation = data["donation"],
            id = data["id"],
            email = data["email"],
            created_at = data["created_at"],
            source = data["source"],
            currency = data["currency"],
            price = data["price"],
            sale_rate = data["sale_rate"],
            game_id = data["game_id"]
        )


from typing import Union
from itchio.DownloadKey import DownloadKey
from itchio.User import User
from itchio.Purchase import Purchase
from itchio.Session import Session
from itchio.Earnings import Earnings


class Game:
    def __init__(
        self,
        cover_url: str,
        created_at: str,
        downloads_count: int,
        id: int,
        min_price: int,
        p_android: bool,
        p_linux: bool,
        p_osx: bool,
        p_windows: bool,
        published: bool,
        published_at: str,
        purchase_count: Union[int, None],
        short_text: str,
        title: str,
        type: str,
        url: str,
        views_count: int,
        earnings: Union[dict, None],
        session: Session):

        self.cover_url = cover_url
        self.created_at = created_at
        self.downloads_count = downloads_count
        self.id = id
        self.min_price = min_price
        self.p_android = p_android
        self.p_linux = p_linux
        self.p_osx = p_osx
        self.p_windows = p_windows
        self.published = published
        self.published_at = published_at
        self.purchase_count = purchase_count
        self.short_text = short_text
        self.title = title
        self.type = type
        self.url = url
        self.views_count = views_count

        if earnings == None:
            self.earnings = None
        else:
            self.earnings = []
            for entry in earnings:
                self.earnings.append(
                    Earnings.parse_from_dict(entry)
                )

        self.session = session

    def purchases(self, user: Union[str, int, User]) -> list[Purchase]:
        """
        Get a purchases of this Game

        Parameters:
            `user`: Either the id, the User object or the email of the User
        """

        param = ""
        
        if type(user) == User:
            param = "user_id"
            name = User.id
        elif type(user) == str:
            param = "email"
            name = user
        elif type(user) == int:
            param = "user_id"
            name = user
        else:
            raise ValueError(f"Parameter user has either to be type of str, int or User, not {type(user)}")

        data = self.session.get(f"game/{self.id}/purchases?{param}={name}")
        purchases = []
        for purchase in data["purchases"]:
            purchases.append(
                Purchase.parse_from_dict(purchase)
            )

        return purchases

    def download_keys(self, user_or_key: Union[str, int, User]) -> DownloadKey:
        """
        Get a download key of this Game

        Parameters:
            `user_or_key`: Either the id of the User, the email address of the User, the User object or the download key
        """

        param = ""

        if type(user_or_key) == User:
            param = "user_id"
            name  = user_or_key.id
        elif type(user_or_key) == int:
            param = "user_id"
            name = user_or_key
        elif type(user_or_key) == str and "@" in user_or_key:
            param = "email"
            name = user_or_key
        elif type(user_or_key) == str:
            param = "download_key"
            name = user_or_key
        else:
            raise ValueError(f"Parameter user_or_key has either to be type of str, int or User, not {type(user_or_key)}")

        data = self.session.get(f"game/{self.id}/download_keys?{param}={name}")
        if "errors" in data:
            raise Exception(data["errors"])
        else:
            return DownloadKey.parse_from_dict(data["download_key"])

    def sync(self) -> object:
        """
        Sync this object with the information in the API
        """
        data = self.session.get(f"game/{self.id}")
        new_game = self.parse_from_dict(data, self.session)
        self.__dict__.update(new_game.__dict__)


    @staticmethod
    def parse_from_dict(data: dict, session: Session) -> object:
        if "purchase_count" not in data:
            data["purchase_count"] = None
        if "earnings" not in data:
            data["earnings"] = None
        if "downloads_count" not in data:
            data["downloads_count"] = None
        if "published" not in data:
            data["published"] = None
        if "views_count" not in data:
            data["views_count"] = None
        if "title" not in data:
            data["title"] = None

        return Game(
            cover_url = data["cover_url"],
            created_at = data["created_at"],
            downloads_count = data["downloads_count"],
            id = data["id"],
            min_price = data["min_price"],
            p_android = data["p_android"],
            p_linux = data["p_linux"],
            p_osx = data["p_osx"],
            p_windows = data["p_windows"],
            published = data["published"],
            published_at = data["published_at"],
            purchase_count = data["purchase_count"],
            short_text = data["short_text"],
            title = data["title"],
            type = data["type"],
            url = data["url"],
            views_count = data["views_count"],
            earnings = data["earnings"],
            session = session
        )


from itchio.Session import Session
from itchio.User import User
from itchio.UserCollection import UserCollection
from itchio.Game import Game
from itchio.Earnings import Earnings
from itchio.GameCollection import GameCollection


from setuptools import setup

setup(
    name="itchio",
    version="1.1.0",
    description="Interact with the REST-API of itch.io",
    long_description=open("./README.md").read(),
    long_description_content_type="text/markdown",
    license="MIT",
    author="Johannes Pour",
    url="https://github.com/Tch1b0/itchio-lib",
    author_email="Johannes@ben11.de",
    packages=["itchio"],
    install_requires=["requests==2.25.1"]
)


class Earnings:
    def __init__(
        self,
        currency: str,
        amount_formatted: str,
        amount: int):

        self.currency = currency
        self.amount_formatted = amount_formatted
        self.amount = amount

    @staticmethod
    def parse_from_dict(data: dict):
        return Earnings(
            currency = data["currency"],
            amount_formatted = data["amount_formatted"],
            amount = data["amount"]
        )